﻿using System.Linq;

namespace HeuristicLab.Problems.ProgramSynthesis.Base.Erc {
  using System.Collections.Generic;

  using Common;
  using Core;
  using Data;

  using HeuristicLab.Problems.ProgramSynthesis.Base.Erc.Interfaces;
  using Parameters;
  using Persistence.Default.CompositeSerializers.Storable;

  [StorableClass]
  public class FloatVectorErcOptions : ErcOption, IReadOnlyVectorErcOptions<double> {
    private const string ConstantsParameterName = "Constants";

    public FloatVectorErcOptions() : this(new double[0][]) {
    }

    public FloatVectorErcOptions(double[][] constants) {
      IsEnabled = true;
      var mappedConstants = constants.Select(c => new DoubleArray(c));
      Parameters.Add(new ValueParameter<ItemArray<DoubleArray>>(ConstantsParameterName, new ItemArray<DoubleArray>(mappedConstants)));
    }

    [StorableConstructor]
    private FloatVectorErcOptions(bool deserializing) : base(deserializing) { }

    public FloatVectorErcOptions(FloatVectorErcOptions origin, Cloner cloner) : base(origin, cloner) { }

    public IValueParameter<ItemArray<DoubleArray>> ConstantsParameter
    {
      get { return (IValueParameter<ItemArray<DoubleArray>>)Parameters[ConstantsParameterName]; }
    }

    public IEnumerable<IReadOnlyList<double>> Constants
    {
      get { return ConstantsParameter.Value.Select(v => v.Data); }
      set
      {
        var mappedConstants = value.Select(c => new DoubleArray(c.ToArray()));
        ConstantsParameter.Value = new ItemArray<DoubleArray>(mappedConstants);
      }
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new FloatVectorErcOptions(this, cloner);
    }
  }
}
