﻿namespace HeuristicLab.Problems.ProgramSynthesis.Base.Erc {
  using System;
  using System.Collections.Generic;
  using System.Linq;

  using HeuristicLab.Common;
  using HeuristicLab.Core;
  using HeuristicLab.Data;
  using HeuristicLab.Parameters;
  using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
  using HeuristicLab.Problems.ProgramSynthesis.Base.Erc.Interfaces;

  [StorableClass]
  public class IntegerErcOptions : ErcOption, IReadOnlyIntegerErcOptions {
    private const string ConstantsParameterName = "Constants";
    private const string RangeParameterName = "Range";

    public IntegerErcOptions() : this(default(int), default(int)) {
      IsEnabled = false;
    }

    public IntegerErcOptions(int start, int end) : this(start, end, new int[0]) {
    }

    public IntegerErcOptions(int[] constants) : this(default(int), default(int), constants) {
    }

    public IntegerErcOptions(int start, int end, int[] constants) {
      if (start > end) {
        throw new InvalidOperationException("start >= end");
      }

      IsEnabled = true;
      Parameters.Add(new ValueParameter<IntArray>(ConstantsParameterName, new IntArray(constants)));
      Parameters.Add(new FixedValueParameter<IntRange>(RangeParameterName, new IntRange(start, end)));
    }

    [StorableConstructor]
    private IntegerErcOptions(bool deserializing) : base(deserializing) { }

    public IntegerErcOptions(IntegerErcOptions origin, Cloner cloner) : base(origin, cloner) { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new IntegerErcOptions(this, cloner);
    }

    public IValueParameter<IntArray> ConstantsParameter
    {
      get { return (IValueParameter<IntArray>)Parameters[ConstantsParameterName]; }
    }

    public IReadOnlyList<int> Constants
    {
      get { return ConstantsParameter.Value.Data; }
      set { ConstantsParameter.Value = new IntArray(value.ToArray()); }
    }

    public IValueParameter<IntRange> RangeParameter
    {
      get { return (IValueParameter<IntRange>)Parameters[RangeParameterName]; }
    }

    /// <summary>
    ///     The minimum INTEGER that will be produced as an ephemeral random INTEGER constant or from a call to INTEGER.RAND.
    /// </summary>
    public int Start
    {
      get { return RangeParameter.Value.Start; }
      set { RangeParameter.Value.Start = value; }
    }

    /// <summary>
    ///     The maximum INTEGER that will be produced as an ephemeral random INTEGER constant or from a call to INTEGER.RAND.
    /// </summary>
    public int End
    {
      get { return RangeParameter.Value.End; }
      set { RangeParameter.Value.End = value; }
    }
  }
}
