﻿using System.Linq;

namespace HeuristicLab.Problems.ProgramSynthesis.Base.Erc {
  using System.Collections.Generic;

  using Common;
  using Core;
  using Data;

  using HeuristicLab.Problems.ProgramSynthesis.Base.Erc.Interfaces;
  using Parameters;
  using Persistence.Default.CompositeSerializers.Storable;

  [StorableClass]
  public class StringVectorErcOptions : ErcOption, IReadOnlyVectorErcOptions<string> {
    private const string ConstantsParameterName = "Constants";

    public StringVectorErcOptions() : this(new string[0][]) {
    }

    public StringVectorErcOptions(string[][] constants) {
      IsEnabled = true;
      var mappedConstants = constants.Select(c => new StringArray(c));
      Parameters.Add(new ValueParameter<ItemArray<StringArray>>(ConstantsParameterName, new ItemArray<StringArray>(mappedConstants)));
    }

    [StorableConstructor]
    private StringVectorErcOptions(bool deserializing) : base(deserializing) { }

    public StringVectorErcOptions(StringVectorErcOptions origin, Cloner cloner) : base(origin, cloner) { }

    public IValueParameter<ItemArray<StringArray>> ConstantsParameter
    {
      get { return (IValueParameter<ItemArray<StringArray>>)Parameters[ConstantsParameterName]; }
    }

    public IEnumerable<IReadOnlyList<string>> Constants
    {
      get { return ConstantsParameter.Value.Select(v => v.ToArray()); }
      set
      {
        var mappedConstants = value.Select(c => new StringArray(c.ToArray()));
        ConstantsParameter.Value = new ItemArray<StringArray>(mappedConstants);
      }
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new StringVectorErcOptions(this, cloner);
    }
  }
}
